use ratatui::DefaultTerminal;

use crate::{lobby::Lobby, message::{Message, MessageKind}, player::Player, ui::ui};

#[derive(Debug, Default)]
pub struct Client {
    pub addr: String,
    pub client: reqwest::Client,
    pub user: Option<Player>,
    pub name: String,
    pub exit: bool,
    pub popup: bool,
    pub popup_title: String,
    pub popup_content: String,
    pub response: String,
    pub lobby: Option<Lobby>,
}

impl Client {
    pub async fn run(&mut self, terminal: &mut DefaultTerminal) -> anyhow::Result<()> {
        self.addr = "http://127.0.0.1:8080".to_string();
        self.client = reqwest::Client::new();

        while !self.exit {
            terminal.draw(|frame| ui(frame, self))?;
            crate::ui::handle_events(self).await?;
        }

        Ok(())
    }

    pub fn exit(&mut self) {
        self.exit = true;
    }

    pub async fn send_player(&mut self, player: &Player, addr: &str) -> anyhow::Result<()> {
        let player_message = Message::new(
            &player.get_name(),
            MessageKind::CreatePlayer,
            player.encode()?
        )?;

        let response = self.client
            .post([&addr, "/create/player"].concat())
            .header("Content-Type", "application/json")
            .body(player_message.encode()?)
            .send().await;

        if let Err(e) = &response {
            self.response = e.to_string();
            return Ok(())
        } else {
            self.response = format!("{:?}", response?);
        }

        Ok(())
    }
}
